@@ -25,17 +25,17 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-document-loader.c,v 1.5.2.1 2006/04/08 14:46:31 pborelli Exp $
+ * $Id: gedit-document-loader.c,v 1.9 2006/08/25 09:30:34 pborelli Exp $
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#include <sys/mman.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#include <sys/mman.h>
 #include <string.h>
 #include <errno.h>
 
@@ -275,7 +275,7 @@ update_document_contents (GeditDocumentLoader  *loader,
 			  gint                  file_size,
 			  GError              **error)
 {
-	gedit_debug (DEBUG_SAVER);
+	gedit_debug (DEBUG_LOADER);
 
 	g_return_val_if_fail (file_size > 0, FALSE);
 	g_return_val_if_fail (file_contents != NULL, FALSE);
@@ -543,7 +543,7 @@ load_local_file_real (GeditDocumentLoader *loader)
 				    
 		if (mapped_file == MAP_FAILED)
 		{
-			gedit_debug_message (DEBUG_SAVER, "mmap failed");
+			gedit_debug_message (DEBUG_LOADER, "mmap failed");
 
 			result = gnome_vfs_result_from_errno ();
 
@@ -618,7 +618,7 @@ static void
 load_local_file (GeditDocumentLoader *loader,
 		 const gchar         *fname)
 {
-	gedit_debug (DEBUG_SAVER);
+	gedit_debug (DEBUG_LOADER);
 
 	g_signal_emit (loader,
 		       signals[LOADING],
@@ -644,10 +644,10 @@ load_local_file (GeditDocumentLoader *loader,
 				    
 		return;			    
 	}
-	
+
 	g_free (loader->priv->local_file_name);
 	loader->priv->local_file_name = g_strdup (fname);
-	
+
 	g_timeout_add_full (G_PRIORITY_HIGH,
 			    0,
 			    (GSourceFunc) load_local_file_real,
@@ -734,10 +734,18 @@ async_read_cb (GnomeVFSAsyncHandle *handle,
 	/* end of the file, we are done! */
 	if (bytes_read == 0 || result != GNOME_VFS_OK)
 	{
-		update_document_contents (loader,
-					  loader->priv->buffer,
-					  loader->priv->bytes_read,
-					  &loader->priv->error);
+		if (loader->priv->bytes_read == 0)
+		{
+			if (loader->priv->encoding == NULL)
+				loader->priv->auto_detected_encoding = gedit_encoding_get_current ();
+		}
+		else
+		{
+			update_document_contents (loader,
+						  loader->priv->buffer,
+						  loader->priv->bytes_read,
+						  &loader->priv->error);
+		}
 
 		remote_load_completed_or_failed (loader);
 
