@@ -27,7 +27,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-document.c,v 1.132 2006/05/18 22:55:16 pborelli Exp $
+ * $Id: gedit-document.c,v 1.135 2006/08/24 13:17:13 paolo Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -94,7 +94,6 @@ struct _GeditDocumentPrivate
 	gint	     last_save_was_manually : 1; 	
 	gint	     language_set_by_user : 1;
 	gint         is_saving_as : 1;
-	gint         has_selection : 1;
 	gint         stop_cursor_moved_emission : 1;
 
 	gchar	    *uri;
@@ -138,7 +137,6 @@ enum {
 	PROP_READ_ONLY,
 	PROP_ENCODING,
 	PROP_CAN_SEARCH_AGAIN,
-	PROP_HAS_SELECTION,
 	PROP_ENABLE_SEARCH_HIGHLIGHTING
 };
 
@@ -297,11 +295,9 @@ gedit_document_get_property (GObject    *object,
 		case PROP_CAN_SEARCH_AGAIN:
 			g_value_set_boolean (value, gedit_document_get_can_search_again (doc));
 			break;
-		case PROP_HAS_SELECTION:
-			g_value_set_boolean (value, doc->priv->has_selection);
-			break;
 		case PROP_ENABLE_SEARCH_HIGHLIGHTING:
 			g_value_set_boolean (value, gedit_document_get_enable_search_highlighting (doc));
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -350,22 +346,6 @@ gedit_document_mark_set (GtkTextBuffer     *buffer,
 									       iter,
 									       mark);
 
-	if (mark == gtk_text_buffer_get_insert (buffer) ||
-	    mark == gtk_text_buffer_get_selection_bound (buffer))
-	{
-		gboolean has_selection;
-
-		has_selection = gtk_text_buffer_get_selection_bounds (buffer,
-								      NULL,
-								      NULL);
-
-		if (has_selection != doc->priv->has_selection)
-		{
-			doc->priv->has_selection = has_selection;
-			g_object_notify (G_OBJECT (doc), "has-selection");
-		}
-	}
-
 	if (mark == gtk_text_buffer_get_insert (buffer))
 	{
 		emit_cursor_moved (doc);
@@ -430,16 +410,6 @@ gedit_document_class_init (GeditDocumentClass *klass)
 					 		       FALSE,
 					 		       G_PARAM_READABLE));
 
-	/* This has been properly moved in GtkTextBuffer in gtk 2.10, so when
-	 * we switch to 2.10 we can remove it and part of with gedit_document_mark_set.
-	 */
-	g_object_class_install_property (object_class, PROP_HAS_SELECTION,
-					 g_param_spec_boolean ("has-selection",
-					 		       "Has selection",
-					 		       "Wheter the document has selected text",
-					 		       FALSE,
-					 		       G_PARAM_READABLE));
-
 	g_object_class_install_property (object_class, PROP_ENABLE_SEARCH_HIGHLIGHTING,
 					 g_param_spec_boolean ("enable-search-highlighting",
 					 		       "Enable Search Highlighting",
@@ -612,7 +582,6 @@ gedit_document_init (GeditDocument *doc)
 
 	doc->priv->readonly = FALSE;
 
-	doc->priv->has_selection = FALSE;
 	doc->priv->stop_cursor_moved_emission = FALSE;
 
 	doc->priv->last_save_was_manually = TRUE;
@@ -743,7 +712,7 @@ set_uri (GeditDocument *doc,
 
 			if (strcmp (doc->priv->mime_type, "text/plain") != 0)
 			{
-				language = gtk_source_languages_manager_get_language_from_mime_type (
+				language = gedit_languages_manager_get_language_from_mime_type (
 							gedit_get_languages_manager (),
 							doc->priv->mime_type);
 			}
@@ -993,10 +962,10 @@ document_loader_loaded (GeditDocumentLoader *loader,
 
 	/* special case creating a named new doc */
 	else if (doc->priv->create &&
-	         (error->code == GNOME_VFS_ERROR_NOT_FOUND))
+	         (error->code == GNOME_VFS_ERROR_NOT_FOUND) &&
+	         (gedit_utils_uri_has_file_scheme (doc->priv->uri)))
 	{
 		reset_temp_loading_data (doc);
-		// FIXME: do other stuff??
 
 		g_signal_emit (doc,
 			       document_signals[LOADED],
@@ -1246,14 +1215,6 @@ gedit_document_get_deleted (GeditDocument *doc)
 	return !gnome_vfs_uri_exists (doc->priv->vfs_uri);
 }
 
-gboolean
-_gedit_document_get_has_selection (GeditDocument *doc)
-{
-	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), FALSE);
-
-	return doc->priv->has_selection;
-}
-
 /*
  * If @line is bigger than the lines of the document, the cursor is moved
  * to the last line and FALSE is returned.
