@@ -27,7 +27,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-utils.c,v 1.75.2.1 2006/04/20 16:53:41 pborelli Exp $
+ * $Id: gedit-utils.c,v 1.79 2006/08/24 13:17:13 paolo Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -60,22 +60,27 @@
 
 #define STDIN_DELAY_MICROSECONDS 100000
 
+/* Returns true if uri is a file: uri and is not a chained uri */
 gboolean
 gedit_utils_uri_has_file_scheme (const gchar *uri)
 {
-	gchar *canonical_uri = NULL;
+	gchar *canonical_uri;
+	gchar *tmp;
 	gboolean res;
 
 	canonical_uri = gnome_vfs_make_uri_canonical (uri);
 	g_return_val_if_fail (canonical_uri != NULL, FALSE);
 
-	res = g_str_has_prefix (canonical_uri, "file:");
-
+	tmp = gnome_vfs_get_local_path_from_uri (canonical_uri);
+	res = (tmp != NULL);
+	
 	g_free (canonical_uri);
+	g_free (tmp);
 
 	return res;
 }
 
+/* FIXME: we should check for chained URIs */
 gboolean
 gedit_utils_uri_has_writable_scheme (const gchar *uri)
 {
@@ -134,6 +139,56 @@ gedit_utils_menu_position_under_widget (GtkMenu  *menu,
 	*push_in = TRUE;
 }
 
+void
+gedit_utils_menu_position_under_tree_view (GtkMenu  *menu,
+					   gint     *x,
+					   gint     *y,
+					   gboolean *push_in,
+					   gpointer  user_data)
+{
+	GtkTreeView *tree = GTK_TREE_VIEW (user_data);
+	GtkTreeModel *model;
+	GtkTreeSelection *selection;
+	GtkTreeIter iter;
+	
+	model = gtk_tree_view_get_model (tree);
+	g_return_if_fail (model != NULL);
+
+	selection = gtk_tree_view_get_selection (tree);
+	g_return_if_fail (selection != NULL);
+
+	if (gtk_tree_selection_get_selected (selection, NULL, &iter))
+	{
+		GtkTreePath *path;
+		GdkRectangle rect;
+
+		gdk_window_get_origin (GTK_WIDGET (tree)->window, x, y);
+			
+		path = gtk_tree_model_get_path (model, &iter);
+		gtk_tree_view_get_cell_area (tree, path,
+					     gtk_tree_view_get_column (tree, 0), /* FIXME 0 for RTL ? */
+					     &rect);
+		gtk_tree_path_free (path);
+		
+		*x += rect.x;
+		*y += rect.y + rect.height;
+		
+		if (gtk_widget_get_direction (GTK_WIDGET (tree)) == GTK_TEXT_DIR_RTL)
+		{
+			GtkRequisition requisition;
+			gtk_widget_size_request (GTK_WIDGET (menu), &requisition);
+			*x += rect.width - requisition.width;
+		}
+	}
+	else
+	{
+		/* no selection -> regular "under widget" positioning */
+		gedit_utils_menu_position_under_widget (menu,
+							x, y, push_in,
+							tree);
+	}
+}
+
 GtkWidget *
 gedit_gtk_button_new_with_stock_icon (const gchar *label,
 				      const gchar *stock_id)
@@ -466,12 +521,16 @@ gedit_utils_get_stdin (void)
 void 
 gedit_warning (GtkWindow *parent, const gchar *format, ...)
 {
-	va_list args;
-	gchar *str;
-	GtkWidget *dialog;
-
+	va_list         args;
+	gchar          *str;
+	GtkWidget      *dialog;
+	GtkWindowGroup *wg = NULL;
+	
 	g_return_if_fail (format != NULL);
 
+	if (parent != NULL)
+		wg = parent->group;
+		
 	va_start (args, format);
 	str = g_strdup_vprintf (format, args);
 	va_end (args);
@@ -483,16 +542,22 @@ gedit_warning (GtkWindow *parent, const gchar *format, ...)
 		   	GTK_BUTTONS_OK,
 			str);
 
+	g_free (str);
+	
+	if (wg != NULL)
+		gtk_window_group_add_window (wg, GTK_WINDOW (dialog));
+		
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 
 	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 
-	gtk_dialog_run (GTK_DIALOG (dialog));
-
-	gtk_widget_destroy (dialog);
-
-	g_free (str);
+	g_signal_connect (G_OBJECT (dialog),
+			  "response",
+			  G_CALLBACK (gtk_widget_destroy),
+			  NULL);
+			  
+	gtk_widget_show (dialog);
 }
 
 /*
